enum SequentialFocusDirection {
Forward,
Backward,
}Expand description
https://html.spec.whatwg.org/multipage/#sequential-focus-direction
A sequential focus direction is one of two possible values: “forward”, or “backward”. They are used in the below algorithms to describe the direction in which sequential focus travels at the user’s request.
Variants