DocumentEventHandler

Struct DocumentEventHandler 

pub(crate) struct DocumentEventHandler {
Show 14 fields window: Dom<Window>, pending_input_events: DomRefCell<Vec<ConstellationInputEvent>>, mouse_move_event_index: DomRefCell<Option<usize>>, click_counting_info: DomRefCell<ClickCountingInfo>, last_mouse_button_down_point: Cell<Option<Point2D<f32, CSSPixel>>>, down_button_count: Cell<u32>, current_hover_target: MutNullableDom<Element>, most_recently_clicked_element: MutNullableDom<Element>, most_recent_mousemove_point: Cell<Option<Point2D<f32, CSSPixel>>>, current_cursor: Cell<Option<Cursor>>, active_touch_points: DomRefCell<Vec<Dom<Touch>>>, active_keyboard_modifiers: Cell<Modifiers>, active_pointer_ids: DomRefCell<FxHashMap<i32, i32>>, next_touch_pointer_id: Cell<i32>,
}
The DocumentEventHandler is a structure responsible for handling input events for the [crate::Document] and storing data related to event handling. It exists to decrease the size of the [crate::Document] structure.

Fields

window: Dom<Window>

The Window element for this DocumentEventHandler.

pending_input_events: DomRefCell<Vec<ConstellationInputEvent>>

Pending input events, to be handled at the next rendering opportunity.

mouse_move_event_index: DomRefCell<Option<usize>>

The index of the last mouse move event in the pending input events queue.

click_counting_info: DomRefCell<ClickCountingInfo>last_mouse_button_down_point: Cell<Option<Point2D<f32, CSSPixel>>>down_button_count: Cell<u32>

The number of currently down buttons, used to decide which kind of pointer event to dispatch on MouseDown/MouseUp.

current_hover_target: MutNullableDom<Element>

The element that is currently hovered by the cursor.

most_recently_clicked_element: MutNullableDom<Element>

The element that was most recently clicked.

most_recent_mousemove_point: Cell<Option<Point2D<f32, CSSPixel>>>

The most recent mouse movement point, used for processing mouseleave events.

current_cursor: Cell<Option<Cursor>>

The currently set Cursor or None if the Document isn’t being hovered by the cursor.

active_touch_points: DomRefCell<Vec<Dom<Touch>>>active_keyboard_modifiers: Cell<Modifiers>

The active keyboard modifiers for the WebView. This is updated when receiving any input event.

active_pointer_ids: DomRefCell<FxHashMap<i32, i32>>

Map from touch identifier to pointer ID for active touch points

next_touch_pointer_id: Cell<i32>

Counter for generating unique pointer IDs for touch inputs

Implementations

Source

impl DocumentEventHandler

pub(crate) fn new(window: &Window) -> Self

pub(crate) fn note_pending_input_event(&self, event: ConstellationInputEvent)

Note a pending input event, to be processed at the next update_the_rendering task.

pub(crate) fn has_pending_input_events(&self) -> bool

Whether or not this [Document] has any pending input events to be processed during “update the rendering.”

pub(crate) fn alternate_action_keyboard_modifier_active(&self) -> bool

pub(crate) fn handle_pending_input_events(&self, can_gc: CanGc)

fn notify_embedder_that_event_was_handled( &self, event: InputEventAndId, result: InputEventResult, )

pub(crate) fn set_cursor(&self, cursor: Option<Cursor>)

fn handle_mouse_left_viewport_event( &self, input_event: &ConstellationInputEvent, mouse_leave_event: &MouseLeftViewportEvent, can_gc: CanGc, )

fn handle_mouse_enter_leave_event( &self, event_target: DomRoot<Node>, related_target: Option<DomRoot<Node>>, event_type: FireMouseEventType, hit_test_result: &HitTestResult, input_event: &ConstellationInputEvent, can_gc: CanGc, )

fn handle_native_mouse_move_event( &self, input_event: &ConstellationInputEvent, can_gc: CanGc, )

fn update_current_hover_target_and_status( &self, new_hover_target: Option<DomRoot<Element>>, )

pub(crate) fn handle_refresh_cursor(&self)

fn handle_native_mouse_button_event( &self, event: MouseButtonEvent, input_event: &ConstellationInputEvent, can_gc: CanGc, )

https://w3c.github.io/uievents/#mouseevent-algorithms Handles native mouse down, mouse up, mouse click.

fn maybe_trigger_click_for_mouse_button_down_event( &self, event: MouseButtonEvent, input_event: &ConstellationInputEvent, hit_test_result: &HitTestResult, element: &Element, can_gc: CanGc, )

fn maybe_show_context_menu( &self, target: &EventTarget, hit_test_result: &HitTestResult, input_event: &ConstellationInputEvent, can_gc: CanGc, )

fn handle_touch_event( &self, event: EmbedderTouchEvent, input_event: &ConstellationInputEvent, can_gc: CanGc, ) -> InputEventResult

fn update_active_touch_points_when_early_return( &self, event: EmbedderTouchEvent, )

Updates the active touch points when a hit test fails early.

  • For Down: No action needed; a failed down event won’t create an active point.
  • For Move: No action needed; position information is unavailable, so we cannot update.
  • For Up/Cancel: Remove the corresponding touch point and its pointer ID mapping.

When a touchup or touchcancel occurs at that touch point, a warning is triggered: Received touchup/touchcancel event for a non-active touch point.

fn handle_keyboard_event( &self, keyboard_event: EmbedderKeyboardEvent, can_gc: CanGc, ) -> InputEventResult

The entry point for all key processing for web content

fn handle_ime_event(&self, event: ImeEvent, can_gc: CanGc) -> InputEventResult

fn handle_wheel_event( &self, event: EmbedderWheelEvent, input_event: &ConstellationInputEvent, can_gc: CanGc, ) -> InputEventResult

fn handle_gamepad_event(&self, gamepad_event: EmbedderGamepadEvent)

fn handle_gamepad_connect( &self, _index: usize, name: String, axis_bounds: (f64, f64), button_bounds: (f64, f64), supported_haptic_effects: GamepadSupportedHapticEffects, )

fn handle_gamepad_disconnect(&self, index: usize)

fn receive_new_gamepad_button_or_axis( &self, index: usize, update_type: GamepadUpdateType, )

pub(crate) fn handle_editing_action( &self, element: Option<DomRoot<Element>>, action: EditingActionEvent, can_gc: CanGc, ) -> InputEventResult

pub(crate) fn fire_clipboard_event( &self, target: Option<DomRoot<Element>>, clipboard_event_type: ClipboardEventType, can_gc: CanGc, ) -> DomRoot<ClipboardEvent>

fn write_content_to_the_clipboard(&self, drag_data_store: &DragDataStore)

pub(crate) fn handle_embedder_scroll_event( &self, scrolled_node: ExternalScrollId, )

Handle a scroll event triggered by user interactions from the embedder. https://drafts.csswg.org/cssom-view/#scrolling-events

pub(crate) fn run_default_keyboard_event_handler(&self, event: &KeyboardEvent)

pub(crate) fn do_keyboard_scroll(&self, scroll: KeyboardScroll)

fn get_or_create_pointer_id_for_touch(&self, touch_id: i32) -> i32

Get or create a pointer ID for the given touch identifier. Returns the pointer ID to use for this touch.

fn remove_pointer_id_for_touch(&self, touch_id: i32)

Remove the pointer ID mapping for the given touch identifier.

fn is_primary_pointer(&self, pointer_id: i32) -> bool

Check if this is the primary pointer (for touch events). The first touch to make contact is the primary pointer.